/*
 * This file is part of vospace-rest
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
package it.inaf.oats.vospace;

import it.inaf.ia2.aa.data.User;
import it.inaf.oats.vospace.datamodel.collections.NodeCollection;
import it.inaf.oats.vospace.datamodel.collections.NodeCollectionsWrapper;
import it.inaf.oats.vospace.datamodel.collections.NodeDetailsWrapper;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @author Nicola Fulvio Calabria <nicola.calabria at inaf.it>
 */
@RestController
public class CollectionsController {

    private static final Logger LOG = LoggerFactory.getLogger(CollectionsController.class);

    @Autowired
    private CollectionsService collectionsService;

    // list collections owned by user
    @GetMapping(value = "/collections")
    public ResponseEntity<NodeCollectionsWrapper> listCollections(
            HttpServletRequest request, User principal) {
        LOG.debug("list collections called for user {}", principal.getName());

        NodeCollectionsWrapper ncl = new NodeCollectionsWrapper();

        ncl.setNodeCollections(
                collectionsService.listCollections(principal.getName()));

        return ResponseEntity.ok(ncl);
    }

    // create a new collection with specified title
    @PutMapping(value = "/collections")
    public ResponseEntity<String> createCollection(
            @RequestBody NodeCollection nc, User principal) {
        LOG.debug("create collection called with name {} called for user {}", 
                nc.getTitle(), principal.getName());       

        collectionsService.createNewCollection(nc.getTitle(), principal.getName());

        return ResponseEntity.ok("Collection created");
    }

    // delete collection by id
    @DeleteMapping(value = "/collections/{collectionId}")
    public ResponseEntity<String> deleteCollection(
            @PathVariable("collectionId") Long id, User principal) {
        LOG.debug("delete collection called with id {} for user {}", 
                id, principal.getName());
        
        collectionsService.deleteCollectionById(id, principal.getName());
        
        // TODO: manage case collection not found or request by someone who isn't 
        // the owner
        return ResponseEntity.ok("Collection deleted");
                
    }
    
    // add node to collection
    @PutMapping(value = "/addnode/{collectionId}")
    public ResponseEntity<String> addNodeToCollection(
            @PathVariable("collectionId") Long collectionId,
            @RequestParam("vosPath") String vosPath,
            User principal) {
        LOG.debug("add node {} to collection {} called for user {}",
                vosPath, collectionId, principal.getName()
                );
        
        collectionsService.addNodeToCollection(vosPath, collectionId, 
                principal.getName(), principal.getGroups());
        
        return ResponseEntity.ok("Node added");
    }
    
    // add node to collection
    @DeleteMapping(value = "/removenode/{collectionId}")
    public ResponseEntity<String> removeNodeFromCollection(
            @PathVariable("collectionId") Long collectionId,
            @RequestParam("vosPath") String vosPath,
            User principal) {
        LOG.debug("remove node {} from collection {} called for user {}",
                vosPath, collectionId, principal.getName()
                );
        
        collectionsService.removeNodeFromCollection(vosPath, collectionId, 
                principal.getName(), principal.getGroups());
        
        return ResponseEntity.ok("Node added");
    }
    
    @GetMapping(value = "/getnodesincollection/{collectionId}")
    public ResponseEntity<NodeDetailsWrapper> getNodesInCollection(
            @PathVariable("collectionId") Long collectionId,
            User principal)
    {
        LOG.debug("get nodes in collection {} called for user {}", 
                collectionId, principal.getName());
        
        NodeDetailsWrapper ndw = new NodeDetailsWrapper();
        
        ndw.setNodeDetails(
                collectionsService.getNodeDetailsInCollection(
                        collectionId, principal.getName()));
        
        return ResponseEntity.ok(ndw);
    }
    
}
